﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System;
using Newtonsoft.Json;

[Serializable]
public class AssetBundleItem:AssetBundleItemBase
{
    [SerializeField]
    private string _name;
    [SerializeField]
    private List<AssetBundleSubItem> _items = new List<AssetBundleSubItem>();

    private bool _foldout = false;
    private string _filter = "*.*";

    public AssetBundleItem()
    {

    }

    public AssetBundleItem(string n)
    {
        _name = n;
    }

    public AssetBundleSubItem[] GetSubItems()
    {
        return _items.ToArray();
    }

    public void SetName(string n)
    {
        _name = n;
    }

    public string GetName()
    {
        return _name;
    }

    public string[] GetAssetNames()
    {
        List<string> names = new List<string>();
        for(int i = 0; i < _items.Count; ++i)
        {
            _items[i].GetAssetNames(ref names);
        }

        return names.ToArray();
    }

    public void AddItem(string path)
    {
        AssetBundleSubItem sub = new AssetBundleSubItem(path, _filter);
        sub.SetParentItem(this);
        _items.Add(sub);
    }

    public void RemoveItem(AssetBundleSubItem item)
    {
        _items.Remove(item);
    }

    private string GetCurrentSelectPath()
    {
        string path = "";
        if (Selection.activeObject)
        {
            path = AssetDatabase.GetAssetPath(Selection.activeObject);
        }
        return path;
    }

    public string ToSerialize()
    {
        Dictionary<string, string> result = new Dictionary<string, string>();
        result.Add("name", _name);
        result.Add("filter", _filter);
        List<string> itemStr = new List<string>();
        for(int i = 0; i < _items.Count; ++i)
        {
            string s = _items[i].ToSerialize();
            itemStr.Add(s);
        }
        result.Add("item", JsonConvert.SerializeObject(itemStr));
        return JsonConvert.SerializeObject(result);
    }

    public void UnSerialize(string s)
    {
        Dictionary<string, string> result = JsonConvert.DeserializeObject<Dictionary<string, string>>(s);
        if(!result.TryGetValue("name",out _name))
        {
            _name = "";
        }

        if (!result.TryGetValue("filter", out _filter))
        {
            _filter = "*.*";
        }

        string tmp;
        if (result.TryGetValue("item", out tmp))
        {
            List<string> itemStr = JsonConvert.DeserializeObject<List<string>>(tmp);
            for(int i = 0; i < itemStr.Count; ++i)
            {
                AssetBundleSubItem sub = new AssetBundleSubItem();
                sub.SetParentItem(this);
                sub.UnSerialize(itemStr[i]);
                _items.Add(sub);
            }
        }
    }

        public void Draw()
    {
#if UNITY_EDITOR
        GUILayout.BeginHorizontal();
        _foldout = EditorGUILayout.Foldout(_foldout, _items.Count.ToString());
        _name = GUILayout.TextField(_name);
        _filter = GUILayout.TextField(_filter);
        if (GUILayout.Button("X", GUILayout.MaxWidth(30)))
        {
            AssetBundleManager.RemoveItem(this);
        }
        if (GUILayout.Button("+", GUILayout.MaxWidth(30)))
        {
            string path = GetCurrentSelectPath();
            if (path.Equals(string.Empty))
            {
                return;
            }
            AddItem(path);
        }
        GUILayout.EndHorizontal();
        if (_foldout)
        {
            GUILayout.BeginVertical();
            for (int i = 0; i < _items.Count;++i)
            {
                _items[i].Draw();
            }
            GUILayout.EndVertical();
        }

       
        
#endif
    }

    override public void Remove(AssetBundleItemBase it)
    {
        RemoveItem((AssetBundleSubItem)it);
    }

}
