﻿using Newtonsoft.Json;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEngine;

public static class AssetBundleManager
{
    static private List<AssetBundleItem> _items = new List<AssetBundleItem>();
    const string AseetBundleConfig = "AssetBundleConfig";

    static public void AddItem(string name)
    {
        AssetBundleItem it = new AssetBundleItem(name);
        _items.Add(it);
    }

    static public AssetBundleItem GetItemByIndex(int i)
    {
        return _items[i];
    }

    static public void RemoveItem(AssetBundleItem it)
    {
        _items.Remove(it);
    }

    static public void RemoveItemByIndex(int i)
    {
        _items.RemoveAt(i);
    }

    static public AssetBundleItem[] GetItems()
    {
        return _items.ToArray();
    }

    static public void GetAssetInfo(string assetName, AssetBundleSubItem item, List<AssetInfo> list)
    {
        if(item.IsDir())
        {
            AssetBundleSubItem[] subItem = item.GetSubItems();
            for (int i = 0; i < subItem.Length;++i)
            {
                GetAssetInfo(assetName, subItem[i], list);
            }

            return;
        }
        AssetInfo info = new AssetInfo();
        info.ABName = assetName;
        info.DependNames = item.GetDepende();
        info.AssetPath = item.GetResPath();
        info.Name = item.GetFileBaseName();
        list.Add(info);
    }

    static public void GenResourceMap(string rootPath)
    {
        if (Directory.Exists(rootPath) == false)
        {
            Directory.CreateDirectory(rootPath);
        }
        AssetInfoList list = new AssetInfoList();
        List<AssetInfo> assetInfoList = new List<AssetInfo>();
        for (int i = 0; i < _items.Count; ++i)
        {
            AssetBundleSubItem[] subItem = _items[i].GetSubItems();
            for (int j = 0; j < subItem.Length; ++j)
            {
                GetAssetInfo(_items[i].GetName(), subItem[j], assetInfoList);
            }
        }
        list.AssetInfos = assetInfoList.ToArray();

        string jsonStr = JsonUtility.ToJson(list,true);
        File.WriteAllText(rootPath + "/AssetInfoMap.json", jsonStr);
    }

    static public void GenAssetBundle(string rootPath)
    {
        if (Directory.Exists(rootPath) == false)
        {
            Directory.CreateDirectory(rootPath);
        }
        GenResourceMap(rootPath);
        List<AssetBundleBuild> buildList = new List<AssetBundleBuild>();
        for (int i = 0; i < _items.Count; ++i)
        {
            AssetBundleBuild build = new AssetBundleBuild();
            build.assetBundleName = _items[i].GetName();
            build.assetNames = _items[i].GetAssetNames();
            buildList.Add(build);
        }
        BuildPipeline.BuildAssetBundles(rootPath, buildList.ToArray(), BuildAssetBundleOptions.None, EditorUserBuildSettings.activeBuildTarget);
    }

    static public void SaveConfig()
    {
        List<string> itemStr = new List<string>();
        for(int i = 0; i < _items.Count;++i)
        {
            itemStr.Add(_items[i].ToSerialize());
        }

        string jsonString = JsonConvert.SerializeObject(itemStr);
        EditorPrefs.SetString(AseetBundleConfig, jsonString);
        
    }

    static public void LoadConfig()
    {
        string jsonString = EditorPrefs.GetString(AseetBundleConfig);
        List<string> itemStr = JsonConvert.DeserializeObject<List<string>>(jsonString);
        _items.Clear();
        for (int i = 0; i < itemStr.Count; ++i)
        {
            AssetBundleItem it = new AssetBundleItem();
            it.UnSerialize(itemStr[i]);
            _items.Add(it);
        }
        
    }

    static public void Draw()
    {
#if UNITY_EDITOR
        GUILayout.BeginVertical();
        for(int i = 0; i < _items.Count; ++i)
        {
            _items[i].Draw();
        }
        GUILayout.EndVertical();
#endif
    }
}
