﻿using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEngine;
using System;
using Newtonsoft.Json;
using System.Linq;

[Serializable]
public class AssetBundleSubItem:AssetBundleItemBase
{
    [SerializeField]
    private string _resPath;
    [SerializeField]
    private bool _isDir;
    [SerializeField]
    private List<AssetBundleSubItem> _subResItem = new List<AssetBundleSubItem>();
    [SerializeField]
    private int _dependCount;
    [SerializeField]
    private List<string> _dependList = new List<string>();
    private string _filter = "*.*";
    private AssetBundleItemBase _parentItem;

    private bool _foldout = false;
    public AssetBundleSubItem()
    {

    }

    public AssetBundleSubItem(string path,string fiter = "*.*")
    {
        _filter = fiter;
        _dependCount = 0;
        _resPath = path.Replace('\\','/');
        if (File.Exists(_resPath))
        {
            _isDir = false;
            string[] files = AssetDatabase.GetDependencies(_resPath, true);
            for(int i = 0; i < files.Length;++i)
            {
                if(files[i].Replace('\\','/') != _resPath && !IsFilterSucc(files[i].Replace('\\', '/')))
                {
                    _dependList.Add(files[i]);
                    _dependCount++;
                }
            }
        }
        else
        {
            _isDir = true;
        }
        if(_isDir)
        {
            CreateSubItem();
        }
    }

    public AssetBundleSubItem[] GetSubItems()
    {
        return _subResItem.ToArray();
    }

    public string GetResPath()
    {
        return _resPath;
    }

    public void GetAssetNames(ref List<string> names)
    {
        if(!IsDir())
        {
            names.Add(_resPath);
        }
        else
        {
            for(int i = 0; i < _subResItem.Count; ++i)
            {
                _subResItem[i].GetAssetNames(ref names);
            }
        }
    }

    public string GetFileBaseName()
    {
        if(_isDir)
        {
            return "";
        }
        _resPath = _resPath.Replace('\\', '/');
        int pos = _resPath.LastIndexOf('/');
        if(pos > -1)
        {
            return _resPath.Substring(pos + 1);
        }
        return _resPath;
    }

    public bool IsDir()
    {
        return _isDir;
    }

    public void SetParentItem(AssetBundleItemBase p)
    {
        _parentItem = p;
    }

    public void SetFilter(string f)
    {
        _filter = f;
    }

    public bool IsFilterSucc(string path)
    {
        if(_filter.Equals("*.*"))
        {
            return false;
        }

        string[] st = _filter.Split(';');
        for(int i = 0; i < st.Length; ++i)
        {
            if(path.EndsWith(st[i]))
            {
                return false;
            }
        }
        return true;
    }

    public int GetDependCount()
    {
        return _dependCount;
    }

    public string[] GetDepende()
    {
        return _dependList.ToArray();
    }

    public void RemoveSubItem(AssetBundleSubItem item)
    {
        _dependCount -= item.GetDependCount();
        _subResItem.Remove(item);
    }

    private void CreateSubItem()
    {
        string[] files;
        if (_filter != "*.*")
        {
            string[] extensions = _filter.Split(';');
            files = Directory.GetFiles(_resPath, "*.*", SearchOption.TopDirectoryOnly);
            files = files.Where(f => extensions.Contains(System.IO.Path.GetExtension(f).ToLower())).ToArray();
        }
        else
        {
            files = Directory.GetFiles(_resPath, "*.*", SearchOption.TopDirectoryOnly);
        }
        
        for(int i = 0; i < files.Length; ++i)
        {
            if(!files[i].Equals(_resPath))
            {
                AssetBundleSubItem sub = new AssetBundleSubItem(files[i], _filter);
                sub.SetParentItem(this);
                _dependCount += sub.GetDependCount();
                _subResItem.Add(sub);
            }
        }
    }

    public string ToSerialize()
    {
        Dictionary<string, string> result = new Dictionary<string, string>();
        result.Add("resPath", _resPath);
        result.Add("isDir", _isDir.ToString());
        result.Add("dependCount", _dependCount.ToString());
        result.Add("filter", _filter);
        result.Add("dependList", JsonConvert.SerializeObject(_dependList));
        List<string> itemStr = new List<string>();
        for (int i = 0; i < _subResItem.Count; ++i)
        {
            string s = _subResItem[i].ToSerialize();
            itemStr.Add(s);
        }
        result.Add("subResItem", JsonConvert.SerializeObject(itemStr));
        return JsonConvert.SerializeObject(result);
    }

    public void UnSerialize(string s)
    {
        Dictionary<string, string> result = JsonConvert.DeserializeObject<Dictionary<string, string>>(s);
        if(!result.TryGetValue("resPath", out _resPath))
        {
            _resPath = "";
        }

        string tmp;
        if (!result.TryGetValue("isDir", out tmp))
        {
            _isDir = false;
        }
        else
        {
            _isDir = Boolean.Parse(tmp);
        }

        if (!result.TryGetValue("dependCount", out tmp))
        {
            _dependCount = 0;
        }
        else
        {
            _dependCount = int.Parse(tmp);
        }

        if (!result.TryGetValue("filter", out _filter))
        {
            _filter = "*.*";
        }
   

        if (result.TryGetValue("dependList", out tmp))
        {
            _dependList = JsonConvert.DeserializeObject<List<string>>(tmp);
        }

        if (result.TryGetValue("subResItem", out tmp))
        {
            List<string> itemStr = JsonConvert.DeserializeObject<List<string>>(tmp);
            for(int i = 0; i < itemStr.Count; ++i)
            {
                AssetBundleSubItem sub = new AssetBundleSubItem();
                sub.UnSerialize(itemStr[i]);
                sub.SetParentItem(this);
                _subResItem.Add(sub);
            }
        }
    }
    public void Draw()
    {
#if UNITY_EDITOR
        if(_isDir)
        {
            GUILayout.BeginHorizontal();
            _foldout = EditorGUILayout.Foldout(_foldout, _resPath);
            GUILayout.Label("个数：" + _subResItem.Count);
            if (GUILayout.Button("X", GUILayout.MaxWidth(30)))
            {
                if (_parentItem != null)
                {
                    _parentItem.Remove(this);
                }
            }
            GUILayout.EndHorizontal();
            if (_foldout)
            {
                GUILayout.BeginVertical();
                for(int i = 0; i < _subResItem.Count;++i)
                {
                    _subResItem[i].Draw();
                }
                GUILayout.EndVertical();
            }
        }
        else
        {
            GUILayout.BeginHorizontal();
            GUILayout.Label(_resPath);
            if (GUILayout.Button("X",GUILayout.MaxWidth(30)))
            {
                if (_parentItem != null)
                {
                    _parentItem.Remove(this);
                }
            }
            GUILayout.EndHorizontal();
        }
       
#endif
    }

    override public void Remove(AssetBundleItemBase it)
    {
        RemoveSubItem((AssetBundleSubItem)it);
    }
}
